/*
-----------------------------------------------------------------------------
Filename:    TutorialApplication.cpp
-----------------------------------------------------------------------------

This source file is part of the
   ___                 __    __ _ _    _ 
  /___\__ _ _ __ ___  / / /\ \ (_) | _(_)
 //  // _` | '__/ _ \ \ \/  \/ / | |/ / |
/ \_// (_| | | |  __/  \  /\  /| |   <| |
\___/ \__, |_|  \___|   \/  \/ |_|_|\_\_|
	  |___/                              
	  Tutorial Framework
	  http://www.ogre3d.org/tikiwiki/
-----------------------------------------------------------------------------
*/
#include "TutorialApplication.h"

//-------------------------------------------------------------------------------------
TutorialApplication::TutorialApplication(void)
	: mTestNode(NULL)
	, mStopRequested(false)
	, mAngle(0.0f)
{
}

//-------------------------------------------------------------------------------------
TutorialApplication::~TutorialApplication(void)
{
}

//-------------------------------------------------------------------------------------
struct CUSTOMVERTEX
{
	float x,y,z;
	float nx,ny,nz;
	float tu,tv;
};

CUSTOMVERTEX vertex[24] =
{
	// front face:
	{  1.0f, -1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f }, // upper left vertex
	{  1.0f, 1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f },  // upper right vertex
	{  -1.0f, 1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f }, // lower left vertex
	{  -1.0f, -1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f },// lower right vertex
	// right face:
	{  1.0f, 1.0f, -1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f }, // upper left vertex
	{  1.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f },  // upper right vertex
	{  1.0f, -1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f, 1.0f }, // lower left vertex
	{  1.0f, -1.0f, -1.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f },// lower right vertex
	// back face:
	{  1.0f, 1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f }, // upper left vertex
	{  -1.0f, 1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f },  // upper right vertex
	{  -1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f }, // lower left vertex
	{  1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f },// lower right vertex
	// left face
	{  -1.0f, 1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f }, // upper left vertex
	{  -1.0f, 1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f },  // upper right vertex
	{  -1.0f, -1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f }, // lower left vertex
	{  -1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f },// lower right vertex
	// top face:
	{  -1.0f, 1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f }, // upper left vertex
	{  1.0f, 1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f },  // upper right vertex
	{  1.0f, 1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f }, // lower left vertex
	{  -1.0f, 1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f },// lower right vertex
	// floor face
	{  -1.0f, -1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f }, // upper left vertex
	{  1.0f, -1.0f, -1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f },  // upper right vertex
	{  1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f }, // lower left vertex
	{  -1.0f, -1.0f, 1.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f },// lower right vertex
};

WORD index[64] =
{
	0, 1, 2, 2, 3, 0,  
	4, 5, 6, 6, 7, 4,
	8, 9, 10, 10, 11, 8,
	12, 13, 14, 14, 15, 12,
	16, 17, 18, 18, 19, 16,
	20, 21, 22, 22, 23, 20
};

void TutorialApplication::createScene(void)
{
	// Global settings
	mCamera->setPosition(Ogre::Vector3(5.0f, 5.0f, 5.0f));
	mCamera->lookAt(Ogre::Vector3::ZERO);
	Ogre::SceneNode* pLightNode = mSceneMgr->getRootSceneNode()->createChildSceneNode();
	Ogre::Light* directionalLight = mSceneMgr->createLight();
	pLightNode->attachObject(directionalLight);
	directionalLight->setType(Ogre::Light::LT_DIRECTIONAL);
	directionalLight->setDiffuseColour(Ogre::ColourValue::White);
	directionalLight->setSpecularColour(0.1f * Ogre::ColourValue::White);
	directionalLight->setDirection(Ogre::Vector3(50.0f, -50.0f, 50.0f));
	mSceneMgr->setAmbientLight(Ogre::ColourValue::White);

	// Create material "red"
	Ogre::MaterialPtr redMat = Ogre::MaterialManager::getSingleton().create("redMat", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
	redMat->getTechnique(0)->getPass(0)->setLightingEnabled(true);
	redMat->getTechnique(0)->getPass(0)->setAmbient(Ogre::ColourValue::Red);
	redMat->getTechnique(0)->getPass(0)->setDiffuse(Ogre::ColourValue::Red);
	redMat->getTechnique(0)->getPass(0)->setEmissive(Ogre::ColourValue::Red);

	// Calculate size
	Ogre::Vector3 size_min = Ogre::Vector3::ZERO;
	Ogre::Vector3 size_max = Ogre::Vector3::ZERO;
	for(size_t ui = 0; ui < 24; ui++)
	{
		if(vertex[ui].x < size_min.x) size_min.x = vertex[ui].x;
		if(vertex[ui].y < size_min.y) size_min.y = vertex[ui].y;
		if(vertex[ui].z < size_min.z) size_min.z = vertex[ui].z;
		if(vertex[ui].x > size_max.x) size_max.x = vertex[ui].x;
		if(vertex[ui].y > size_max.y) size_max.y = vertex[ui].y;
		if(vertex[ui].z > size_max.z) size_max.z = vertex[ui].z;
	}

	// Create greometry
	Ogre::ManualObject* manual1 = mSceneMgr->createManualObject();
	manual1->setName("manual1");
	manual1->begin("redMat", Ogre::RenderOperation::OT_TRIANGLE_LIST);
	for (size_t i = 0; i < 24 ; i++)
	{
		manual1->position(vertex[i].x, vertex[i].y, vertex[i].z);
		manual1->textureCoord(vertex[i].tu, vertex[i].tv);
		manual1->normal(vertex[i].nx, vertex[i].ny, vertex[i].nz);
	}
	for(size_t i = 0; i < 64; i++) manual1->index(index[i]);
	manual1->end();
	manual1->setLocalAabb(Ogre::Aabb::newFromExtents(size_min, size_max));
	Ogre::MeshPtr pManual1Mesh = manual1->convertToMesh("manual1Mesh");
	mTestNode = mSceneMgr->getRootSceneNode()->createChildSceneNode();
	Ogre::Entity* pEntity = mSceneMgr->createEntity(pManual1Mesh);
	mTestNode->attachObject(pEntity);

	// Create the thread and start work
	assert(!mThread);
	mThread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&TutorialApplication::runThread, this)));
}

//-------------------------------------------------------------------------------------
void TutorialApplication::destroyScene(void)
{
	// Stops the thread
	assert(mThread);
	mStopRequested = true;
	mThread->join();
 
	// Destroy scene
	BaseApplication::destroyScene();
}

bool TutorialApplication::frameStarted(const Ogre::FrameEvent& evt)
{
	boost::unique_lock<boost::mutex> lock(mMutex);
	mTestNode->setOrientation(Ogre::Quaternion(Ogre::Radian(mAngle), Ogre::Vector3::UNIT_Y));
	return true;
}

//-------------------------------------------------------------------------------------
void TutorialApplication::runThread()
{
	while(!mStopRequested)
	{
		boost::unique_lock<boost::mutex>* lock = new boost::unique_lock<boost::mutex>(mMutex);
		mAngle += 0.005f;
		delete lock;
		boost::this_thread::sleep(boost::posix_time::milliseconds(25));
	}
}

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
	INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
#else
	int main(int argc, char *argv[])
#endif
	{
		// Create application object
		TutorialApplication app;

		try
		{
			app.go();
		}
		catch( Ogre::Exception& e )
		{
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
			MessageBox( NULL, e.getFullDescription().c_str(), "An exception has occured!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
			std::cerr << "An exception has occured: " << e.getFullDescription().c_str() << std::endl;
#endif
		}

		return 0;
	}

#ifdef __cplusplus
}
#endif
